@@ -16,7 +16,7 @@ AnyEvent::Handle - non-blocking I/O on file handles via AnyEvent
          warn "got error $msg\n";
          $hdl->destroy;
          $cv->send;
-      );
+      };
 
    # send some request line
    $hdl->push_write ("getinfo\015\012");
@@ -81,7 +81,7 @@ sub _load_func($) {
 
 =over 4
 
-=item $handle = B<new> AnyEvent::TLS fh => $filehandle, key => value...
+=item $handle = B<new> AnyEvent::Handle fh => $filehandle, key => value...
 
 The constructor supports these arguments (all as C<< key => value >> pairs).
 
@@ -712,7 +712,7 @@ Replace the current C<on_stoptls> callback (see the C<on_stoptls> constructor ar
 
 =cut
 
-sub on_starttls {
+sub on_stoptls {
    $_[0]{on_stoptls} = $_[1];
 }
 
@@ -905,11 +905,14 @@ sub push_write {
            ->($self, @_);
    }
 
+   # we downgrade here to avoid hard-to-track-down bugs,
+   # and diagnose the problem earlier and better.
+
    if ($self->{tls}) {
-      $self->{_tls_wbuf} .= $_[0];
+      utf8::downgrade $self->{_tls_wbuf} .= $_[0];
       &_dotls ($self)    if $self->{fh};
    } else {
-      $self->{wbuf}      .= $_[0];
+      utf8::downgrade $self->{wbuf}      .= $_[0];
       $self->_drain_wbuf if $self->{fh};
    }
 }
@@ -1713,7 +1716,7 @@ sub stop_read {
 sub start_read {
    my ($self) = @_;
 
-   unless ($self->{_rw} || $self->{_eof}) {
+   unless ($self->{_rw} || $self->{_eof} || !$self->{fh}) {
       Scalar::Util::weaken $self;
 
       $self->{_rw} = AE::io $self->{fh}, 0, sub {
@@ -1816,6 +1819,7 @@ sub _dotls {
    while (length ($tmp = Net::SSLeay::BIO_read ($self->{_wbio}))) {
       $self->{wbuf} .= $tmp;
       $self->_drain_wbuf;
+      $self->{tls} or return; # tls session might have gone away in callback
    }
 
    $self->{_on_starttls}
@@ -1924,7 +1928,7 @@ sub starttls {
 
 Shuts down the SSL connection - this makes a proper EOF handshake by
 sending a close notify to the other side, but since OpenSSL doesn't
-support non-blocking shut downs, it is not guarenteed that you can re-use
+support non-blocking shut downs, it is not guaranteed that you can re-use
 the stream afterwards.
 
 =cut
@@ -1932,7 +1936,7 @@ the stream afterwards.
 sub stoptls {
    my ($self) = @_;
 
-   if ($self->{tls}) {
+   if ($self->{tls} && $self->{fh}) {
       Net::SSLeay::shutdown ($self->{tls});
 
       &_dotls;
@@ -2019,6 +2023,29 @@ sub AnyEvent::Handle::destroyed::AUTOLOAD {
    #nop
 }
 
+=item $handle->destroyed
+
+Returns false as long as the handle hasn't been destroyed by a call to C<<
+->destroy >>, true otherwise.
+
+Can be useful to decide whether the handle is still valid after some
+callback possibly destroyed the handle. For example, C<< ->push_write >>,
+C<< ->starttls >> and other methods can call user callbacks, which in turn
+can destroy the handle, so work can be avoided by checking sometimes:
+
+   $hdl->starttls ("accept");
+   return if $hdl->destroyed;
+   $hdl->push_write (...
+
+Note that the call to C<push_write> will silently be ignored if the handle
+has been destroyed, so often you can just ignore the possibility of the
+handle being destroyed.
+
+=cut
+
+sub destroyed { 0 }
+sub AnyEvent::Handle::destroyed::destroyed { 1 }
+
 =item AnyEvent::Handle::TLS_CTX
 
 This function creates and returns the AnyEvent::TLS object used by default
