@@ -34,7 +34,7 @@ sub timer {
    EV::timer $arg{after}, $arg{interval}, $arg{cb}
 }
 
-*AE::io = defined &EV::_ae_io
+*AE::io = defined &EV::_ae_io # 3.8
    ? \&EV::_ae_io
    : sub($$$) { EV::io $_[0], $_[1] ? EV::WRITE : EV::READ, $_[2] };
 
@@ -65,10 +65,6 @@ sub child {
    }
 }
 
-*AE::child = sub($$) {
-   EV::child $_[0], 0, $_[1]
-};
-
 sub idle {
    my ($class, %arg) = @_;
 
