@@ -106,6 +106,7 @@ you want to use the pure-perl backend.
 package AnyEvent::Impl::Perl;
 
 use Scalar::Util qw(weaken);
+use List::Util ();
 
 use AnyEvent (); BEGIN { AnyEvent::common_sense }
 use AnyEvent::Util ();
@@ -119,17 +120,17 @@ sub MAXWAIT() { 3600 } # never sleep for longer than this many seconds
 BEGIN {
    local $SIG{__DIE__};
    my $time_hires = eval "use Time::HiRes (); 1";
-   my $clk_tck    = eval "use POSIX (); POSIX::sysconf (&POSIX::_SC_CLK_TCK)";
+   my $clk_tck    = eval "use POSIX (); POSIX::sysconf (POSIX::_SC_CLK_TCK ())";
    my $round; # actual granularity
 
-   if ($time_hires && eval "&Time::HiRes::clock_gettime (&Time::HiRes::CLOCK_MONOTONIC)") {
+   if ($time_hires && eval "&Time::HiRes::clock_gettime (Time::HiRes::CLOCK_MONOTONIC ())") {
       warn "AnyEvent::Impl::Perl: using CLOCK_MONOTONIC as timebase.\n" if $AnyEvent::VERBOSE >= 8;
       *_update_clock = sub {
          $NOW  = &Time::HiRes::time;
          $MNOW = Time::HiRes::clock_gettime (&Time::HiRes::CLOCK_MONOTONIC);
       };
 
-   } elsif (100 <= $clk_tck && $clk_tck <= 1000000 && "use POSIX (); (POSIX::times())[0] != -1") { # -1 is also a valid return value :/
+   } elsif (100 <= $clk_tck && $clk_tck <= 1000000 && eval { (POSIX::times ())[0] != -1 }) { # -1 is also a valid return value :/
       warn "AnyEvent::Impl::Perl: using POSIX::times (monotonic) as timebase.\n" if $AnyEvent::VERBOSE >= 8;
       my $HZ1 = 1 / $clk_tck;
 
@@ -310,7 +311,7 @@ sub AE::timer($$$) {
 
    if ($interval) {
       $self = [$MNOW + $after , sub {
-         $_[0][0] = $MNOW + $interval;
+         $_[0][0] = List::Util::max $_[0][0] + $interval, $MNOW;
          push @timer, $_[0];
          weaken $timer[-1];
          $need_sort = $_[0][0] if $_[0][0] < $need_sort;
